/* mbed Microcontroller Library
 * Copyright (c) 2006-2013 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "mbed.h"
#include "cc3000.h"
#include "main.h"
#include "xively.h"
#include "xi_err.h"

#define XI_FEED_ID 123 // set Xively Feed ID (numerical, no quoutes)
#define XI_API_KEY "apikey" // set Xively API key (double-quoted string) 

using namespace mbed_cc3000;

tUserFS user_info;
DigitalOut myled(LED1);
Ticker flipper_seconds;
Ticker flipper_led;
uint32_t time_seconds;

/* cc3000 module declaration specific for user's board. Check also init() */
#if (MY_BOARD == WIGO)
cc3000 wifi(PTA16, PTA13, PTD0, SPI(PTD2, PTD3, PTC5), PORTA_IRQn);
Serial pc(USBTX, USBRX);
#elif (MY_BOARD == WIFI_DIPCORTEX)
cc3000 wifi(p28, p27, p30, SPI(p21, p14, p37), PIN_INT0_IRQn);
Serial pc(UART_TX, UART_RX);
#else

#endif

/**
 *  \brief Print cc3000 information
 *  \param none
 *  \return none
 */
void print_cc3000_info() {
    uint8_t myMAC[8];

    printf("MAC address + cc3000 info \r\n");
    wifi.get_user_file_info((uint8_t *)&user_info, sizeof(user_info));
    wifi.get_mac_address(myMAC);
    printf(" MAC address %02x:%02x:%02x:%02x:%02x:%02x \r\n \r\n", myMAC[0], myMAC[1], myMAC[2], myMAC[3], myMAC[4], myMAC[5]);

    printf(" FTC        %i \r\n",user_info.FTC);
    printf(" PP_version %i.%i \r\n",user_info.PP_version[0], user_info.PP_version[1]);
    printf(" SERV_PACK  %i.%i \r\n",user_info.SERV_PACK[0], user_info.SERV_PACK[1]);
    printf(" DRV_VER    %i.%i.%i \r\n",user_info.DRV_VER[0], user_info.DRV_VER[1], user_info.DRV_VER[2]);
    printf(" FW_VER     %i.%i.%i \r\n",user_info.FW_VER[0], user_info.FW_VER[1], user_info.FW_VER[2]);
}

/**
 *  \brief Connect to SSID with a timeout
 *  \param ssid     Name of SSID
 *  \param key      Password
 *  \param sec_mode Security mode
 *  \return none
 */
void connect_to_ssid(char *ssid, char *key, unsigned char sec_mode) {
    printf("Connecting to SSID: %s. Timeout is 10s. \r\n",ssid);
    if (wifi.connect_to_AP((uint8_t *)ssid, (uint8_t *)key, sec_mode) == true) {
        printf(" Connected. \r\n");
    } else {
        printf(" Connection timed-out (error). Please restart. \r\n");
        while(1);
  }
}

/**
 *  \brief Connect to SSID without security
 *  \param ssid Name of SSID
 *  \return none
 */
void connect_to_ssid(char *ssid) {
    wifi.connect_open((uint8_t *)ssid);
}

/**
 *  \brief Count seconds
 *  \param none
 *  \return none
 */
void update_timer() {
    time_seconds++;
}

/**
 *  \brief xively demo
 *  \param  none
 *  \return int
 */
int main() {
    init(); /* board dependent init */
    pc.baud(115200);

    wifi.start(0);
    printf("cc3000 xively demo. \r\n");
    print_cc3000_info();

    printf("Attempting SSID Connection. \r\n");
    wifi._wlan.ioctl_set_connection_policy(0, 0, 0);
#ifndef CC3000_TINY_DRIVER
#ifdef AP_KEY
    connect_to_ssid(SSID, AP_KEY, AP_SECURITY);
#else
    connect_to_ssid(SSID);
#endif
#else
    connect_to_ssid(SSID);
#endif
    printf("DHCP request \r\n");
    while (wifi.is_dhcp_configured() == false) {
         wait_ms(500);
         printf("  Waiting for dhcp to be set. \r\n");
    }

    tNetappIpconfigRetArgs ipinfo2;
    wifi.get_ip_config(&ipinfo2); // data is returned in the ipinfo2 structure
    printf("DHCP assigned IP Address = %d.%d.%d.%d \r\n", ipinfo2.aucIP[3], ipinfo2.aucIP[2], ipinfo2.aucIP[1], ipinfo2.aucIP[0]);

    xi_feed_t feed;
    printf("Size of xi_feed_t: %d \r\n", sizeof(xi_feed_t)); //with default settings this can get over 11kB
    std::memset(&feed, 0, sizeof( xi_feed_t ) );
   
    feed.feed_id = XI_FEED_ID;
    feed.datastream_count = 2;
    
    feed.datastreams[0].datapoint_count = 1;
    xi_datastream_t* status_datastream = &feed.datastreams[0];
    strcpy(status_datastream->datastream_id, "LED1");
    xi_datapoint_t* led_status = &status_datastream->datapoints[0];

    feed.datastreams[1].datapoint_count = 1;
    xi_datastream_t* counter_datastream = &feed.datastreams[1];
    strcpy(counter_datastream->datastream_id, "Uptime");
    xi_datapoint_t* counter = &counter_datastream->datapoints[0];
    
    xi_context_t* xi_context = xi_create_context(XI_HTTP, XI_API_KEY, feed.feed_id);

    if (xi_context == NULL)
    {
        printf("Context failed to initialized. \r\n");
        return -1;
    }
        
    flipper_seconds.attach(&update_timer, 1.0);

    while(1) {  
        xi_set_value_f32(led_status, myled);  
        xi_set_value_f32(counter, time_seconds);
      
        printf( "update...\r\n");
        xi_feed_update(xi_context, &feed);
        printf( "done...\r\n");
        wait(10);
        myled = myled ^ 0x1;
    }

}

